/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Nicolas Pessemier
 Contributor(s): Romain Rouvoy
 =============================================================================*/
package org.objectweb.fractal.spoonlet.binding;

import static org.objectweb.fractal.fraclet.types.Cardinality.COLLECTION;
import static org.objectweb.fractal.spoonlet.binding.BindingHelper.bindingName;

import java.util.List;

import org.objectweb.fractal.fraclet.annotations.Requires;

import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;
import spoon.template.Template;

/**
 * An annotation processor to manage the Fraclet annotation
 * <code>Requires</code>.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class RequiresProcessor extends AbstractBindingProcessor<Requires> {
    /* (non-Javadoc)
     * @see spoon.processing.AbstractProcessor#init()
     */
    @Override
    public void init() {
        addProcessedAnnotationType(Requires.class);
        super.init();
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.spoonlet.binding.BindingProcessor#insertMethodBodies(spoon.reflect.declaration.CtClass, spoon.reflect.declaration.CtField, java.lang.annotation.Annotation)
     */
    protected void insertMethodBodies(CtClass<?> c, CtField<?> f, Requires a) {
        String name = bindingName(f, a);
        Template tmpl = new BindingSingletonTemplate(f, name);
        if (COLLECTION.equals(a.cardinality()))
            tmpl = new BindingCollectionTemplate(f, name);
        insertBegin(tmpl, c, "listFc", List.class);
        insertBegin(tmpl, c, "bindFc", String.class, Object.class);
        insertBegin(tmpl, c, "unbindFc", String.class);
        insertBegin(tmpl, c, "lookupFc", String.class);
    }
}