/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.adl.spoonlet.attribute;

import static java.util.logging.Level.FINER;
import static java.util.logging.Logger.getLogger;
import static org.objectweb.fractal.adl.spoonlet.definition.AbstractDefinitionProcessor.addArgument;
import static org.objectweb.fractal.adl.spoonlet.definition.Arguments.asArgument;
import static org.objectweb.fractal.adl.spoonlet.definition.DefinitionGenerator.primitive;
import static org.objectweb.fractal.spoonlet.attribute.AttributeHelper.attributeController;
import static org.objectweb.fractal.spoonlet.attribute.AttributeHelper.attributeName;
import static org.objectweb.fractal.spoonlet.attribute.AttributeHelper.attributeValue;
import static org.objectweb.fractal.spoonlet.attribute.AttributeHelper.controllerFullname;
import static org.objectweb.fractal.spoonlet.utils.Constants.LOG;

import java.util.logging.Logger;

import org.objectweb.fractal.adl.spoonlet.definition.DefinitionTags;
import org.objectweb.fractal.document.Element;
import org.objectweb.fractal.fraclet.annotations.Attribute;

import spoon.processing.AbstractAnnotationProcessor;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtElement;
import spoon.reflect.declaration.CtField;
import spoon.reflect.reference.CtTypeReference;

/**
 * Processor interpreting the annotation <code>Attribute</code> to generate
 * the associated <code>attribute</code> elements in Fractal ADL definitions.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class AttributeProcessor extends
        AbstractAnnotationProcessor<Attribute, CtField<?>> implements
        AtributeTags, DefinitionTags {
    private final Logger log = getLogger("spoonlet.fractaladl.attribute");

    /* (non-Javadoc)
     * @see spoon.processing.AbstractProcessor#init()
     */
    @Override
    public void init() {
        addProcessedAnnotationType(Attribute.class);
        super.init();
    }

    /* (non-Javadoc)
     * @see spoon.processing.AbstractAnnotationProcessor#inferConsumedAnnotationType()
     */
    @Override
    public boolean inferConsumedAnnotationType() {
        return false;
    }

    /* (non-Javadoc)
     * @see spoon.processing.AnnotationProcessor#process(java.lang.annotation.Annotation, spoon.reflect.declaration.CtElement)
     */
    public void process(Attribute ann, CtField<?> fld) {
        CtClass<?> comp = (CtClass<?>) fld.getParent();
        Element root = primitive().definition(comp);
        String name = attributeName(fld, ann);
        if (LOG && log.isLoggable(FINER))
            log.finer("FractalADL - Processing attribute "
                    + root.getName() + "." + attributeName(fld, ann) + "...");
        Element attributes = null;
        for (Element child : root.getChilds())
            if (ATTRIBUTES_TAG.equals(child.getName())) {
                attributes = child;
                break;
            }
        if (attributes == null) { // Create the attributes element if needed.
            attributes = new Element(ATTRIBUTES_TAG);
            root.getChilds().add(attributes);
            attributes.setAttribute(ATTR_SIGNATURE,
                    controllerFullname(attributeController(comp)));
            for (CtTypeReference<?> ref = comp.getSuperclass(); ref != null;) {
                CtClass<?> parent = (CtClass<?>) ref.getDeclaration();
                ref = parent.getSuperclass();
                if (parent == null)
                    continue;
                for (CtElement attr : parent
                        .getAnnotatedChildren(Attribute.class)) {
                    Attribute a = attr.getAnnotation(Attribute.class);
                    attributes.getChilds().add(
                            getAttribute(a, (CtField<?>) attr));
                    addArgument(root, name,
                            attributeValue((CtField<?>) attr, a));
                }
            }
        }
        attributes.getChilds().add(getAttribute(ann, fld));
        addArgument(root, name, attributeValue(fld, ann));
    }

    /**
     * Provides the element associated to the attribute.
     * 
     * @param a the annotation processed.
     * @param f the field processed.
     * @return the element associated to the attribute.
     */
    public static final Element getAttribute(Attribute a, CtField<?> f) {
        Element attr = new Element(ATTRIBUTE_TAG);
        attr.setAttribute(ATTR_NAME, attributeName(f, a));
        attr.setAttribute(ATTR_VALUE, asArgument.apply(attributeName(f, a)));
        return attr;
    }
}